"""
Observicia - Policy-Aware Tracing SDK for LLM Applications
"""

from observicia.core.context_manager import ObservabilityContext
from observicia.core.policy_engine import PolicyEngine, PolicyResult, Policy
from observicia.core.tracing_manager import TracingClient
from observicia.core.token_tracker import TokenTracker
from observicia.core.patch_manager import PatchManager
from typing import List, Optional, Dict, Any
import os
import yaml

__version__ = "0.1.9"


def init() -> None:
    """Initialize the Observicia SDK."""
    # Get the config file path from an environment variable
    config_file = os.environ.get("OBSERVICIA_CONFIG_FILE",
                                 "observicia_config.yaml")

    # Default logging configuration
    default_logging = {
        "file": None,
        "telemetry": {
            "enabled": False,
            "format": "json"
        },
        "messages": {
            "enabled": False,
            "level": "INFO"
        },
        "chat": {
            "enabled": False,
            "level": "none",
            "file": None
        }
    }

    try:
        with open(config_file, "r") as file:
            config = yaml.safe_load(file)

            # Extract configurations
            service_name = config.get("service_name", "default-service")
            otel_endpoint = config.get("otel_endpoint", None)
            opa_endpoint = config.get("opa_endpoint", None)
            policies = config.get("policies", [])
            logging_config = config.get("logging", default_logging)

            policy_objects = [Policy(**policy)
                              for policy in policies] if policies else None

            ObservabilityContext.initialize(service_name=service_name,
                                            otel_endpoint=otel_endpoint,
                                            opa_endpoint=opa_endpoint,
                                            policies=policy_objects,
                                            logging_config=logging_config)

            # Auto-detect and patch installed providers
            patch_manager = PatchManager()
            patch_manager.patch_all()

    except FileNotFoundError:
        print(f"Configuration file {config_file} not found. Using defaults.")
        ObservabilityContext.initialize(service_name="default-service",
                                        logging_config=default_logging)
    except yaml.YAMLError as e:
        print(
            f"Error parsing the YAML configuration file: {e}. Using defaults.")
        ObservabilityContext.initialize(service_name="default-service",
                                        logging_config=default_logging)


# Expose main decorators
from observicia.core.tracing_manager import trace, trace_rag, trace_stream

# Expose utilities
from observicia.utils.helpers import get_current_span, get_current_context

__all__ = [
    "init", "trace", "trace_rag", "trace_stream", "get_current_span",
    "get_current_context", "PolicyResult", "__version__"
]



import sqlite3
from typing import Dict, Any
from datetime import datetime
from opentelemetry.sdk.trace import ReadableSpan
from opentelemetry.sdk.trace.export import SpanExporter
from opentelemetry.trace import SpanContext


class SQLiteSpanExporter(SpanExporter):
    """SpanExporter that writes spans to a SQLite database."""

    def __init__(self, database_path: str):
        """Initialize SQLite exporter with database path."""
        self.database_path = database_path
        self._initialize_database()

    def _initialize_database(self) -> None:
        """Create the database schema if it doesn't exist."""
        with sqlite3.connect(self.database_path) as conn:
            cursor = conn.cursor()
            cursor.execute('''
                CREATE TABLE IF NOT EXISTS telemetry (
                    id INTEGER PRIMARY KEY AUTOINCREMENT,
                    timestamp TEXT NOT NULL,
                    transaction_id TEXT,
                    user_id TEXT,
                    model TEXT,
                    provider TEXT,
                    request_type TEXT,
                    prompt_tokens INTEGER,
                    completion_tokens INTEGER,
                    total_tokens INTEGER,
                    duration_ms REAL,
                    success BOOLEAN,
                    trace_id TEXT,
                    span_id TEXT,
                    parent_span_id TEXT
                )
            ''')
            conn.commit()

    def _extract_span_data(self, span: ReadableSpan) -> Dict[str, Any]:
        """Extract relevant data from a span for database insertion."""
        ctx: SpanContext = span.get_span_context()
        attributes = span.attributes or {}

        # Calculate duration in milliseconds
        duration_ms = (span.end_time - span.start_time
                       ) / 1_000_000  # Convert nanoseconds to milliseconds

        return {
            'timestamp':
            datetime.utcfromtimestamp(span.start_time / 1e9).isoformat(),
            'transaction_id':
            attributes.get('transaction_id', ''),
            'user_id':
            attributes.get('user.id', ''),
            'model':
            attributes.get('llm.model', ''),
            'provider':
            attributes.get('llm.provider', ''),
            'request_type':
            attributes.get('llm.request.type', ''),
            'prompt_tokens':
            attributes.get('prompt.tokens', 0),
            'completion_tokens':
            attributes.get('completion.tokens', 0),
            'total_tokens':
            attributes.get('total.tokens', 0),
            'duration_ms':
            duration_ms,
            'success':
            attributes.get('policy.passed', True),
            'trace_id':
            format(ctx.trace_id, '032x'),
            'span_id':
            format(ctx.span_id, '016x'),
            'parent_span_id':
            format(span.parent.span_id, '016x') if span.parent else ''
        }

    def export(self, spans: list[ReadableSpan]) -> None:
        """Export the spans to SQLite database."""
        try:
            with sqlite3.connect(self.database_path) as conn:
                cursor = conn.cursor()

                for span in spans:
                    # Only process completion spans with token data
                    if 'completion' in span.name and 'prompt.tokens' in span.attributes:
                        span_data = self._extract_span_data(span)

                        cursor.execute(
                            '''
                            INSERT INTO telemetry (
                                timestamp, transaction_id, user_id, model, provider,
                                request_type, prompt_tokens, completion_tokens,
                                total_tokens, duration_ms, success, trace_id,
                                span_id, parent_span_id
                            ) VALUES (
                                :timestamp, :transaction_id, :user_id, :model, :provider,
                                :request_type, :prompt_tokens, :completion_tokens,
                                :total_tokens, :duration_ms, :success, :trace_id,
                                :span_id, :parent_span_id
                            )
                        ''', span_data)

                conn.commit()
            return None
        except Exception as e:
            print(f"Error exporting spans to SQLite: {e}")
            return None

    def force_flush(self, timeout_millis: float = 30000) -> bool:
        """Force flush the exporter."""
        return True

    def shutdown(self) -> None:
        """Shutdown the exporter."""
        pass



"""
Logging utilities for Observicia SDK.
"""

import logging
import sys
from datetime import datetime
from typing import Any, Dict, Optional, Union, Literal, Sequence, TYPE_CHECKING
import json
from opentelemetry import trace
from opentelemetry.trace import SpanContext
from opentelemetry._logs import set_logger_provider
from opentelemetry.sdk._logs import LoggerProvider
from opentelemetry.sdk._logs.export import BatchLogRecordProcessor, ConsoleLogExporter
from opentelemetry.sdk.trace.export import SpanExporter
from opentelemetry.sdk.trace import ReadableSpan

if TYPE_CHECKING:
    from ..core.context_manager import ObservabilityContext


class FileSpanExporter(SpanExporter):
    """SpanExporter that writes spans to a file in JSON format."""

    def __init__(self, file_path: str):
        self.file_path = file_path
        # Configure file logger for spans
        self.logger = logging.getLogger("SpanExporter")
        self.logger.setLevel(logging.INFO)

        # Create file handler
        file_handler = logging.FileHandler(file_path)
        file_handler.setFormatter(
            logging.Formatter('%(message)s')  # Raw message for JSON
        )
        self.logger.addHandler(file_handler)

        # Remove any existing handlers to avoid duplicate output
        for handler in self.logger.handlers[:-1]:
            self.logger.removeHandler(handler)

    def _extract_span_data(self, span: ReadableSpan) -> Dict[str, Any]:
        """Extract relevant data from a span for serialization."""
        ctx = span.get_span_context()

        # Convert attributes to serializable format
        attributes = {}
        for key, value in span.attributes.items():
            if isinstance(value, (str, int, float, bool)):
                attributes[key] = value
            else:
                attributes[key] = str(value)

        # Extract events
        events = []
        for event in span.events:
            event_data = {
                "name": event.name,
                "timestamp": event.timestamp,
                "attributes": {
                    k: str(v) if not isinstance(v,
                                                (str, int, float, bool)) else v
                    for k, v in event.attributes.items()
                }
            }
            events.append(event_data)

        # Build span data structure
        span_data = {
            "type": "span",
            "timestamp": datetime.utcnow().isoformat(),
            "name": span.name,
            "trace_id": format(ctx.trace_id, "032x"),
            "span_id": format(ctx.span_id, "016x"),
            "parent_id":
            format(span.parent.span_id, "016x") if span.parent else None,
            "start_time": span.start_time,
            "end_time": span.end_time,
            "attributes": attributes,
            "status": {
                "status_code": span.status.status_code.name,
                "description": span.status.description
            },
            "events": events
        }

        return span_data

    def export(self, spans: Sequence[ReadableSpan]) -> None:
        """Export the spans to file."""
        try:
            for span in spans:
                span_data = self._extract_span_data(span)
                self.logger.info(json.dumps(span_data))
            return None
        except Exception as e:
            print(f"Error exporting spans to file: {e}")
            return None

    def force_flush(self, timeout_millis: float = 30000) -> bool:
        """Force flush the exporter."""
        return True

    def shutdown(self) -> None:
        """Shutdown the exporter."""
        for handler in self.logger.handlers:
            handler.close()


class JsonFormatter(logging.Formatter):
    """Custom JSON formatter for main logging."""

    def __init__(self, service_name: str):
        super().__init__()
        self.service_name = service_name

    def format(self, record: logging.LogRecord) -> str:
        log_data = {
            'timestamp': datetime.fromtimestamp(record.created).isoformat(),
            'level': record.levelname,
            'service': self.service_name,
            'message': record.getMessage(),
            'trace_context': getattr(record, 'trace_context', {}),
            'extra': getattr(record, 'extra', {})
        }
        return json.dumps(log_data)


class ChatFormatter(logging.Formatter):
    """Custom JSON formatter for chat logging."""

    def __init__(self, service_name: str):
        super().__init__()
        self.service_name = service_name

    def format(self, record: logging.LogRecord) -> str:
        """Format the log record into JSON."""
        # Build base log data
        log_data = {
            'timestamp': datetime.fromtimestamp(record.created).isoformat(),
            'service': self.service_name,
            'interaction_type': getattr(record, 'interaction_type', 'unknown'),
            'content': record.getMessage(),
            'metadata': getattr(record, 'metadata', {})
        }

        return json.dumps(log_data)


class ObserviciaLogger:
    """Logger class that supports file-based logging and OpenTelemetry integration."""

    def __init__(self,
                 service_name: str,
                 logging_config: Dict[str, Any],
                 context: Optional['ObservabilityContext'] = None):
        """
        Initialize the logger with the new configuration structure.
        
        Args:
            service_name: Name of the service
            logging_config: Logging configuration dictionary containing telemetry, messages, and chat settings
            context: ObservabilityContext instance for transaction tracking
        """
        self.service_name = service_name
        self.logger = logging.getLogger(service_name)
        self._context = context or ObservabilityContext.get_current()

        # Configure based on messages settings
        messages_config = logging_config.get("messages", {})
        if messages_config.get("enabled", True):
            self.logger.setLevel(
                getattr(logging, messages_config.get("level", "INFO")))
        else:
            self.logger.setLevel(logging.CRITICAL)  # Effectively disable

        # Clear any existing handlers
        self.logger.handlers = []

        # Main log file handler
        if logging_config.get("file"):
            file_handler = logging.FileHandler(logging_config["file"])
            file_handler.setFormatter(JsonFormatter(service_name))
            self.logger.addHandler(file_handler)

        # Configure chat logging
        chat_config = logging_config.get("chat", {})
        if chat_config.get("enabled") and chat_config.get("file"):
            self.chat_logger = logging.getLogger(f"{service_name}_chat")
            self.chat_logger.setLevel(logging.INFO)
            chat_handler = logging.FileHandler(chat_config["file"])
            chat_handler.setFormatter(ChatFormatter(service_name))
            self.chat_logger.addHandler(chat_handler)
        else:
            self.chat_logger = None

        self.chat_level = chat_config.get("level", "none")

        # Configure telemetry if enabled
        telemetry_config = logging_config.get("telemetry", {})
        if telemetry_config.get("enabled", True):
            logger_provider = LoggerProvider()
            set_logger_provider(logger_provider)

            if logging_config.get("file"):
                telemetry_handler = BatchLogRecordProcessor(
                    FileSpanExporter(logging_config["file"]))
                logger_provider.add_log_record_processor(telemetry_handler)

    def _get_trace_context(self) -> Dict[str, str]:
        """Get current trace context if available."""
        span = trace.get_current_span()
        if span:
            ctx: SpanContext = span.get_span_context()
            return {
                "trace_id": format(ctx.trace_id, "032x"),
                "span_id": format(ctx.span_id, "016x")
            }
        return {}

    def _get_trace_context(self) -> Dict[str, str]:
        """Get current trace context if available."""
        span = trace.get_current_span()
        if span:
            ctx: SpanContext = span.get_span_context()
            return {
                "trace_id": format(ctx.trace_id, "032x"),
                "span_id": format(ctx.span_id, "016x")
            }
        return {}

    def log_chat_interaction(
            self,
            interaction_type: Literal['prompt', 'completion', 'system'],
            content: str,
            metadata: Optional[Dict[str, Any]] = None) -> None:
        """Log a chat interaction."""
        if not self.chat_logger or interaction_type not in [
                'prompt', 'completion', 'system'
        ]:
            return

        if self.chat_level == 'both' or self.chat_level == interaction_type or interaction_type == 'system':
            complete_metadata = dict(metadata or {})

            # Add interaction type to metadata
            complete_metadata['interaction_type'] = interaction_type

            # Add user ID from context if available
            if hasattr(self, '_context') and self._context:
                user_id = self._context.get_user_id()
                if user_id:
                    complete_metadata['user_id'] = user_id

            # Add transaction info if available
            if hasattr(self, '_context') and self._context:
                active_transactions = self._context.get_active_transactions()
                if active_transactions:
                    current_transaction = next(
                        iter(active_transactions.values()))
                    complete_metadata.update({
                        'transaction_id':
                        current_transaction.id,
                        'transaction_parent_id':
                        current_transaction.parent_id
                    })

            self.chat_logger.info(content,
                                  extra={'metadata': complete_metadata})

    def _log(self,
             level: int,
             message: str,
             extra: Optional[Dict[str, Any]] = None,
             exc_info: Any = None) -> None:
        """Internal logging method with trace context."""
        trace_context = self._get_trace_context()

        extra_dict = {'trace_context': trace_context}
        if extra:
            extra_dict.update(extra)

        self.logger.log(level, message, extra=extra_dict, exc_info=exc_info)

    # Standard logging methods
    def debug(self,
              message: str,
              extra: Optional[Dict[str, Any]] = None) -> None:
        self._log(logging.DEBUG, message, extra)

    def info(self,
             message: str,
             extra: Optional[Dict[str, Any]] = None) -> None:
        self._log(logging.INFO, message, extra)

    def warning(self,
                message: str,
                extra: Optional[Dict[str, Any]] = None) -> None:
        self._log(logging.WARNING, message, extra)

    def error(self,
              message: str,
              extra: Optional[Dict[str, Any]] = None,
              exc_info: Any = None) -> None:
        self._log(logging.ERROR, message, extra, exc_info)

    def critical(self,
                 message: str,
                 extra: Optional[Dict[str, Any]] = None,
                 exc_info: Any = None) -> None:
        self._log(logging.CRITICAL, message, extra, exc_info)



from dataclasses import dataclass, field
from typing import Dict, List, Optional, Set, Literal, Any
from datetime import datetime
from uuid import uuid4

from opentelemetry import trace, baggage
from opentelemetry.trace import Span, SpanKind
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import BatchSpanProcessor
from opentelemetry.exporter.otlp.proto.grpc.trace_exporter import OTLPSpanExporter

from .policy_engine import PolicyEngine, PolicyResult, Policy
from ..utils.logging import FileSpanExporter, ObserviciaLogger
from ..utils.exporter import SQLiteSpanExporter


@dataclass
class Transaction:
    """Represents a logical transaction (e.g. multi-round chat conversation)."""
    id: str
    start_time: datetime
    end_time: Optional[datetime] = None
    metadata: Dict[str, Any] = field(default_factory=dict)
    parent_id: Optional[str] = None


@dataclass
class TraceContext:
    """Core trace context with essential fields"""
    trace_id: str
    parent_id: Optional[str]
    attributes: Dict
    session_id: Optional[str] = None
    user_id: Optional[str] = None
    active_policies: Set[str] = field(default_factory=set)
    violation_count: int = 0
    policy_results: List[PolicyResult] = field(default_factory=list)

    def to_dict(self) -> Dict:
        """Convert context to dictionary for span attributes"""
        return {
            "trace_id": self.trace_id,
            "parent_id": self.parent_id or "",
            "session_id": self.session_id or "",
            "user_id": self.user_id or "",
            "active_policies": list(self.active_policies),
            "violation_count": self.violation_count,
            **self.attributes
        }

    def set_user_id(self, user_id: str) -> None:
        """Set the user ID for this context"""
        self.user_id = user_id

    def add_violation(self, policy_result: PolicyResult) -> None:
        """Record a policy violation"""
        self.violation_count += len(policy_result.violations)
        self.policy_results.append(policy_result)

    def add_policy(self, policy: str) -> None:
        """Add an active policy"""
        self.active_policies.add(policy)

    def remove_policy(self, policy: str) -> None:
        """Remove an active policy"""
        self.active_policies.discard(policy)


class ContextManager:
    """Manages observability context and tracing."""

    def __init__(self,
                 service_name: str,
                 otel_endpoint: Optional[str] = None,
                 opa_endpoint: Optional[str] = None,
                 policies: Optional[List[Policy]] = None,
                 logging_config: Optional[Dict] = None):
        """
        Initialize the context manager.
        
        Args:
            service_name: Name of the service using the SDK
            otel_endpoint: OpenTelemetry endpoint for tracing
            opa_endpoint: OPA server endpoint for policy evaluation
            policies: List of Policy objects defining available policies
            logging_config: Configuration dictionary for logging options
        """
        self._sessions: Dict[str, TraceContext] = {}
        self._service_name = service_name
        self._current_user_id: Optional[str] = None
        self._active_transactions: Dict[str, Transaction] = {}

        # Initialize policy engine if OPA endpoint is provided
        self.policy_engine = PolicyEngine(
            opa_endpoint=opa_endpoint,
            policies=policies) if opa_endpoint else None

        # Use default logging configuration if none provided
        self._logging_config = logging_config or {
            "file": None,
            "sqlite": {
                "enabled": False,
                "database": "observicia.db"
            },
            "telemetry": {
                "enabled": True,
                "format": "json"
            },
            "messages": {
                "enabled": True,
                "level": "INFO"
            },
            "chat": {
                "enabled": False,
                "level": "none",
                "file": None
            }
        }

        # Initialize logger with new configuration
        self._logger = ObserviciaLogger(service_name=service_name,
                                        logging_config=self._logging_config,
                                        context=self)

        # Set up tracing
        provider = TracerProvider()

        if otel_endpoint and self._logging_config["telemetry"]["enabled"]:
            otlp_processor = BatchSpanProcessor(
                OTLPSpanExporter(endpoint=otel_endpoint))
            provider.add_span_processor(otlp_processor)

        # Add file exporter for telemetry if enabled
        if (self._logging_config["file"]
                and self._logging_config["telemetry"]["enabled"]):
            file_processor = BatchSpanProcessor(
                FileSpanExporter(self._logging_config["file"]))
            provider.add_span_processor(file_processor)

        # Add SQLite exporter if enabled
        if (self._logging_config["sqlite"]["enabled"]
                and self._logging_config["sqlite"]["database"]):
            sqlite_processor = BatchSpanProcessor(
                SQLiteSpanExporter(self._logging_config["sqlite"]["database"]))
            provider.add_span_processor(sqlite_processor)

        trace.set_tracer_provider(provider)
        self._tracer = trace.get_tracer(service_name)

    def get_session(self, session_id: str) -> Optional[TraceContext]:
        """Get existing session context"""
        return self._sessions.get(session_id)

    def set_user_id(self, user_id: Optional[str]) -> None:
        """Set the global user ID for all new traces"""
        self._current_user_id = user_id

    def get_user_id(self) -> Optional[str]:
        """Get the current global user ID"""
        return self._current_user_id

    def start_transaction(self,
                          metadata: Optional[Dict[str, Any]] = None,
                          parent_id: Optional[str] = None) -> str:
        """Start a new transaction and return its ID."""
        transaction_id = str(uuid4())
        transaction = Transaction(id=transaction_id,
                                  start_time=datetime.utcnow(),
                                  metadata=metadata or {},
                                  parent_id=parent_id)

        self._active_transactions[transaction_id] = transaction

        if hasattr(self, '_logger'):
            # Log to main logger
            self._logger.info(f"=== Transaction Started: {transaction_id} ===",
                              extra={
                                  'metadata': {
                                      'transaction_id': transaction_id,
                                      'parent_id': parent_id,
                                      'event': 'transaction_start',
                                      **(metadata or {})
                                  }
                              })

            # Log to chat logger
            if hasattr(self._logger,
                       'chat_logger') and self._logger.chat_logger:
                self._logger.log_chat_interaction(
                    interaction_type='system',
                    content=f"=== Begin Transaction: {transaction_id} ===",
                    metadata={
                        'transaction_id': transaction_id,
                        'parent_id': parent_id,
                        'event': 'transaction_start',
                        **(metadata or {})
                    })

        return transaction_id

    def end_transaction(self,
                        transaction_id: str,
                        metadata: Optional[Dict[str, Any]] = None) -> None:
        """End a transaction with the given ID."""
        if transaction_id not in self._active_transactions:
            if hasattr(self, '_logger'):
                self._logger.error(
                    f"Attempt to end non-existent transaction: {transaction_id}"
                )
            raise ValueError(f"Transaction {transaction_id} not found")

        transaction = self._active_transactions[transaction_id]
        transaction.end_time = datetime.utcnow()
        duration = (transaction.end_time -
                    transaction.start_time).total_seconds()

        if metadata:
            transaction.metadata.update(metadata)

        if hasattr(self, '_logger'):
            # Log to main logger
            self._logger.info(f"=== Transaction Ended: {transaction_id} ===",
                              extra={
                                  'metadata': {
                                      'transaction_id': transaction_id,
                                      'parent_id': transaction.parent_id,
                                      'event': 'transaction_end',
                                      'duration_seconds': duration,
                                      **(transaction.metadata or {})
                                  }
                              })

            # Log to chat logger
            if hasattr(self._logger,
                       'chat_logger') and self._logger.chat_logger:
                self._logger.log_chat_interaction(
                    interaction_type='system',
                    content=
                    f"=== End Transaction: {transaction_id} === (Duration: {duration:.2f}s)",
                    metadata={
                        'transaction_id': transaction_id,
                        'parent_id': transaction.parent_id,
                        'event': 'transaction_end',
                        'duration_seconds': duration,
                        **(transaction.metadata or {})
                    })

        del self._active_transactions[transaction_id]

    def get_transaction(self, transaction_id: str) -> Optional[Transaction]:
        """Get transaction details by ID."""
        return self._active_transactions.get(transaction_id)

    def get_active_transactions(self) -> Dict[str, Transaction]:
        """Get all active transactions."""
        return self._active_transactions.copy()

    async def create_span(
        self,
        name: str,
        context: Optional[TraceContext] = None,
        kind: SpanKind = SpanKind.INTERNAL,
        attributes: Optional[Dict] = None,
    ) -> Span:
        """Create a new span with context"""
        span = self._tracer.start_span(name=name,
                                       kind=kind,
                                       attributes={
                                           "service.name": self._service_name,
                                           **(attributes or {})
                                       })

        if context:
            span.set_attributes(context.to_dict())
            baggage.set_baggage("session_id", context.session_id or "")

        return span

    def create_session(self,
                       session_id: str,
                       initial_context: Optional[Dict] = None) -> TraceContext:
        """Create a new session context"""
        context = TraceContext(trace_id=str(session_id),
                               parent_id=None,
                               attributes=initial_context or {},
                               session_id=session_id,
                               user_id=self._current_user_id)
        self._sessions[session_id] = context
        return context

    async def evaluate_policies(
            self,
            context: TraceContext,
            eval_context: Dict,
            policies: Optional[List[str]] = None) -> PolicyResult:
        """Evaluate policies for the given context"""
        if not self.policy_engine:
            return PolicyResult(passed=True, violations=[])

        policies = policies or list(context.active_policies)
        result = await self.policy_engine.evaluate_with_context(
            eval_context=eval_context, policies=policies)

        if not result.passed:
            context.add_violation(result)

        return result


class ObservabilityContext:
    """Global context manager singleton."""

    _instance: Optional[ContextManager] = None

    @classmethod
    def initialize(cls,
                   service_name: str,
                   otel_endpoint: Optional[str] = None,
                   opa_endpoint: Optional[str] = None,
                   policies: Optional[List[Policy]] = None,
                   logging_config: Optional[Dict] = None) -> None:
        """Initialize the global context manager."""
        if cls._instance is None:
            cls._instance = ContextManager(service_name,
                                           otel_endpoint=otel_endpoint,
                                           opa_endpoint=opa_endpoint,
                                           policies=policies,
                                           logging_config=logging_config)

    @classmethod
    def get_current(cls) -> Optional[ContextManager]:
        """Get current context manager instance"""
        if cls._instance is None:
            raise RuntimeError("ObservabilityContext not initialized")
        return cls._instance

    @classmethod
    def create_session(cls,
                       session_id: str,
                       initial_context: Optional[Dict] = None) -> TraceContext:
        """Create a new session context"""
        if cls._instance is None:
            raise RuntimeError("ObservabilityContext not initialized")
        return cls._instance.create_session(session_id, initial_context)

    @classmethod
    def set_user_id(cls, user_id: Optional[str]) -> None:
        """Set the user ID for all new traces"""
        if cls._instance is None:
            raise RuntimeError("ObservabilityContext not initialized")
        cls._instance.set_user_id(user_id)

    @classmethod
    def get_user_id(cls) -> Optional[str]:
        """Get the current user ID"""
        if cls._instance is None:
            raise RuntimeError("ObservabilityContext not initialized")
        return cls._instance.get_user_id()

    @classmethod
    def start_transaction(cls,
                          metadata: Optional[Dict[str, Any]] = None,
                          parent_id: Optional[str] = None) -> str:
        """Start a new transaction."""
        if cls._instance is None:
            raise RuntimeError("ObservabilityContext not initialized")
        return cls._instance.start_transaction(metadata=metadata,
                                               parent_id=parent_id)

    @classmethod
    def end_transaction(cls,
                        transaction_id: str,
                        metadata: Optional[Dict[str, Any]] = None) -> None:
        """End a transaction."""
        if cls._instance is None:
            raise RuntimeError("ObservabilityContext not initialized")
        return cls._instance.end_transaction(transaction_id=transaction_id,
                                             metadata=metadata)

    @classmethod
    def get_transaction(cls, transaction_id: str) -> Optional[Transaction]:
        """Get transaction details."""
        if cls._instance is None:
            raise RuntimeError("ObservabilityContext not initialized")
        return cls._instance.get_transaction(transaction_id)

    @classmethod
    def get_active_transactions(cls) -> Dict[str, Transaction]:
        """Get all active transactions."""
        if cls._instance is None:
            raise RuntimeError("ObservabilityContext not initialized")
        return cls._instance.get_active_transactions()
